/* 3:KrynetiX> can you make a cfg to (asss file) converter? possibly?
 * 29/05/04 - 30/05/04 smong
 * 30/05/04 cyan~fire - open file dialog
 * 20/07/04 smong - added extra checks on periodic, territory and koth
 * 18/02/05 smong - added fg_wz and fg_turf
 * 21/02/05 smong - more intelligent map name generation
 *                  removed absolute path from #include (affects windows only)
 * 24/02/05 smong - spec freq bug fix
 */

#include <stdio.h>
#include <string.h>

#define CFG_QUIET
#define CFG_OPEN_FILE_DIALOG

#define VERSION "1.5"
#define DEFAULT_MAP_NAME "smallmap.lvl"

#ifdef WIN32
# define snprintf _snprintf

# ifdef CFG_OPEN_FILE_DIALOG
#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#  include <commdlg.h>
# define _MAX_PATH 260
# endif
#else /* no dialog on linux */
# ifdef CFG_OPEN_FILE_DIALOG
#  undef CFG_OPEN_FILE_DIALOG
# endif
#endif

#ifdef CFG_OPEN_FILE_DIALOG
static BOOL MakeFilebox(char *path, char *exts);
#endif

static int GetStr(FILE *f, const char *section, const char *key, char *buf, int buflen);
static int GetInt(FILE *f, const char *section, const char *key, int def);

#ifdef CFG_OPEN_FILE_DIALOG
static  BOOL MakeFilebox(char *path, char *exts)
{
	int ret;
	OPENFILENAME ofn;
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = NULL;
	ofn.lpstrFilter = exts;
	ofn.lpstrFile = path;
	ofn.nMaxFile = _MAX_PATH;
	ret = GetOpenFileName(&ofn);
	if (!ret)
		ret -= CommDlgExtendedError();
	return ret;
}
#endif

static int GetStr(FILE *f, const char *section_, const char *key, char *buf, int buflen)
{
	char section[32], line[768], *t;
	int len;

#ifdef CFG_DEBUG
	int linenum = 1;
#endif

	if (!section_ || !key)
	{
		printf("internal error: null pointer\n");
		return 0;
	}

	snprintf(section, 29, "[%s]", section_);
	len = strlen(section);
	if (len > 31) len = 31;

	/* find section */
	rewind(f);
	for(;;)
	{
		if (!fgets(line, 768, f))
		{
#ifndef CFG_QUIET
			printf("could not find %s\n", section);
#endif
			return 0;
		}

#ifdef CFG_DEBUG
		printf("%03d: %.768s", linenum++, line);
#endif

		if (strncmp(line, section, len))
			continue;

#ifdef CFG_DEBUG
		printf(">> found: %s\n", section);
#endif

		break;
	}

	/* find key */
	len = strlen(key);
	for(;;)
	{
		if (!fgets(line, 768, f))
		{
#ifndef CFG_QUIET
			printf("could not find %s:%s\n", section_, key);
#endif
			return 0;
		}

#ifdef CFG_DEBUG
		printf("%03d: %.768s", linenum++, line);
#endif

		if (strncmp(line, key, len))
			continue;

#ifdef CFG_DEBUG
		printf(">> found: %s:%s\n", section_, key);
#endif
		break;
	}

	/* remove cr/lf */
	if ((t = strchr(line, 0x0A))) *t = 0;
	if ((t = strchr(line, 0x0D))) *t = 0;

	t = strchr(line, '=');
	if (t && (t - line) < 768)
	{
		/* copy value to buf */
		t++;
		strncpy(buf, t, buflen);
		buf[ buflen - 1 ] = '\0';
	}

	if (!strlen(buf))
	{
#ifndef CFG_QUIET
		printf("%s:%s has no value\n", section_, key);
#endif
		return 0;
	}

	return 1;
}

static int GetInt(FILE *f, const char *section, const char *key, int def)
{
	char buf[16];
	int value;

	if (GetStr(f, section, key, buf, 16))
	{
		buf[15] = '\0';
		sscanf(buf, "%d", &value);
		return value;
	}

	return def;
}

int main(int argc, char **argv)
{
	FILE *cfg, *conf;
	int balls, flagmode, flagmax, flagmin, virtual, specfreq;
	char mapname[16];
	char levelfiles[256];
	char cfgname[32];

#ifdef CFG_OPEN_FILE_DIALOG
	char path[256];	//argv[1] will point here - Cyan
	memset(path, 0, sizeof(path));	//gotta be clear for MakeFilebox()
#endif

	printf("cfg2conf v%s\n", VERSION);
	printf("  copyright 2004 - 2005 smong     (toktok.sscentral.com)\n");
	printf("  copyright 2004        cyan~fire (www.digitization.org)\n");

	if (argc < 2)
	{
#ifdef CFG_OPEN_FILE_DIALOG
		if (MakeFilebox(path, "subgame Config Files (*.cfg)\0*.cfg\0"
			"All Files (*.*)\0*.*\0"))
		{
			char *t = strrchr(path, '\\');
			if (t)
			{
				strncpy(cfgname, t + 1, sizeof(cfgname));
			    cfgname[ sizeof(cfgname) - 1 ] = 0;
			}
			else
			{
				strncpy(cfgname, path, sizeof(cfgname));
			    cfgname[ sizeof(cfgname) - 1 ] = 0;
			}

			/* crap code, but leaves smong's code alone, mostly */
			argv[1] = path;
		}
		else
#endif
		{
			printf("usage:   %s <server.cfg> [<mapname.lvl>]\n", argv[0]);
			printf("example: %s svs.cfg alpha.lvl\n", argv[0]);
			return 1;
		}
	}

	/* open cfg */
	cfg = fopen(argv[1], "r");
	if (!cfg)
	{
		printf("cannot open %s.\n", argv[1]);
		return 1;
	}

	/* cache/generate some stuff */
	if (argc > 2)
	{
		printf("using '%s' as map name.\n", argv[2]);
		strncpy(mapname, argv[2], 16);
	}
	else if (GetStr(cfg, "Notes", "MapName", mapname, 16))
	{
		if (!strstr(mapname, ".lvl"))
		{
		    strncat(mapname, ".lvl", 16);
		    /* done later
		    mapname[15] = '\0';
			*/
		}
		printf("using Notes:Mapname as map name (%s).\n", mapname);
	}
	else
	{
		/* use cfg name */
		char *t = strchr(argv[1], '.');
		if (t && argv[1] - t < 16)
		{
			printf("generating map name from .cfg name.\n");
			strncpy(mapname, cfgname, 16);
			t = strchr(mapname, '.');
			t[1] = 'l';
			t[2] = 'v';
			t[3] = 'l';
		}
		else
		{
			printf("using default map name (%s).\n", DEFAULT_MAP_NAME);
			strncpy(mapname, DEFAULT_MAP_NAME, 16);
		}
	}
	mapname[15] = '\0';

	/* 0=dropped flags are un-owned, carry all flags to win
	 * 1=dropped flags are owned, own all flags to win
	 * 2=Turf style flag game
	 */
	flagmode = GetInt(cfg, "Flag", "FlagMode", -1);

	/* check flagcount (ctf) */
	flagmax = GetInt(cfg, "Flag", "MaxFlags", 0);
	flagmin = GetInt(cfg, "Flag", "RandomFlags", 0) ? 1 : flagmax;
	virtual = GetInt(cfg, "Prize", "MinimumVirtual", 256);

	balls = GetInt(cfg, "Soccer", "BallCount", 0);


	/* write to arena.conf */
	conf = fopen("arena.conf", "w");
	/* note argv[1] intentionally used instead of cfgname */
	fprintf(conf, "; put this file and %s in arenas/<arena name of your choice>\n\n", argv[1]);
	fprintf(conf, "#include %s\n\n", cfgname);

	/* general */
	fprintf(conf, "[General]\nMap = %s\n", mapname);
	if (GetStr(cfg, "Misc", "LevelFiles", levelfiles, 256))
		fprintf(conf, "LevelFiles = %s\n", levelfiles);
	fprintf(conf, "\n");

	/* spec freq bug fix */
	specfreq = GetInt(cfg, "Team", "SpectatorFrequency", 8025);
	if (specfreq < 0 || specfreq > 9999)
	{
		fprintf(conf, "[Team]\nSpectatorFrequency = 8025\n\n");
	}

	/* modules */
	fprintf(conf, "[Modules]\nAttachModules = \\\n\tfm_normal");

	if (GetInt(cfg, "Misc", "SaveSpawnScore", 1))
		fprintf(conf, " \\\n\tpoints_kill");
	if (flagmax)
		fprintf(conf, " \\\n\tpoints_flag");
	if (balls)
		fprintf(conf, " \\\n\tpoints_goal");
	if ((GetInt(cfg, "Periodic", "RewardDelay", 0) && GetInt(cfg, "Periodic", "RewardPoints", 0)) ||
		(GetInt(cfg, "Territory", "RewardDelay", 0) && GetInt(cfg, "Territory", "RewardPoints", 0)))
		fprintf(conf, " \\\n\tpoints_periodic");
	if (flagmode == 1)
		fprintf(conf, " \\\n\tfg_wz");
	if (flagmode == 2)
		fprintf(conf, " \\\n\tfg_turf");
	if (GetInt(cfg, "King", "ExpireTime", 0) && GetInt(cfg, "King", "RewardFactor", 0))
		fprintf(conf, " \\\n\tkoth");
	fprintf(conf, "\n\n");

	/* flag */
	if (flagmode == 2)
	{
		/* turf */
		fprintf(conf, "Flag:GameType = $FLAGGAME_TURF\n\n");
	}
	else if (flagmax)
	{
		fprintf(conf, "Flag:GameType = $FLAGGAME_BASIC\n");
		fprintf(conf, "Flag:FlagCount = %d - %d\n", flagmin, flagmax);
		fprintf(conf, "Flag:SpawnRadius = %d\n\n", virtual);
	}

	if (balls)
		fprintf(conf, "Soccer:SpawnRadius = %d\n\n", virtual);

	fprintf(conf, "; changed settings:\n\n");

	/* cleanup */
	fclose(cfg);
	fclose(conf);

	return 0;
}

